require 'ftools'
def printUsage
	scriptName = File.basename($0)
	puts "Usage: ruby #{scriptName} [input file] [output file] [fuzz percentage]"
	puts "example: ruby #{scriptName} input.txt output.txt 50"
	Process.exit
end



if (ARGV.size()!=3)
	printUsage
end

inputFile=ARGV[0]
outputFile=ARGV[1]

if (File.exist?(inputFile)==false)
	puts "Error, input file not found!"
	printUsage
end

begin
	fuzzPercentage = Float(ARGV[2])
	rescue
	puts "Error, invalid fuzz percentage, please enter a value between 0 and 100!"
	printUsage
end

	
if (fuzzPercentage>100 or fuzzPercentage<0)
	puts "Error, fuzz percentage out of range, please enter a value between 0 and 100!"
	printUsage
end

# find out the size of the input file in bytes
inputSize =  File.size(inputFile)

# figure out how many bytes are needed to be fuzzed
numBytesToFuzz = inputSize * (fuzzPercentage/100) 

# round to nearest integer
numBytesToFuzz = numBytesToFuzz.round.to_i/1

#prints some information to stdout
puts "Input file size: "+inputSize.to_s
puts "Fuzz percentage: "+fuzzPercentage.to_s()+"%"
puts "Bytes to Fuzz: "+numBytesToFuzz.to_s

File.copy(inputFile,outputFile)

# create hash table to store location of bytes that have been hashed
byteHash={}

# create new file to output
fh = File.new(outputFile, "r+")

for i in (1..numBytesToFuzz)
	# find a random byte to fuzz, check against hash table to make sure it's not already in hash
	begin
		byteToFuzz = rand(inputSize) 
	end while byteHash[byteToFuzz]!=nil
	
	puts "fuzzing byte number "+byteToFuzz.to_s
	
	#store hashed byte number in hash table
	byteHash.store(byteToFuzz,1)
	
	# go to the correct posistion in file
	fh.sysseek(byteToFuzz, IO::SEEK_SET)
	
	# read the data to a file
	
	aByte = fh.sysread(1)
	
	puts aByte


	aByte.each_byte do |c|
		#complement the read byte
		c=[(c&0xFF).to_s]
		
		# go back 1 position in the file
		fh.sysseek(byteToFuzz, IO::SEEK_SET)
		
		# write the complemented data
		fh.syswrite(c.pack("b*"))
	end
	
	
end

#close file handler
fh.close
	
